/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgObject.cpp,v 2.8 2000/05/13 01:46:14 jeff Exp $

*******************************************************************************/


#include <qfile.h>
#include <qtextstream.h>
#include <qfiledlg.h>

#include "pixmap/Pixmap.h"
#include "Dialog.h"
#include "Project.h"
#include "DlgObject.h"
#include "DlgParser.h"
#include "xml/QtArchExpat.h"

QIntDict<char> DlgObject::itsExtensionDict;


/* Some of this code is not really used for now:
 *   - Enter/Leave Project are aware that a object could be in different 
 *     projects. This isn't implemented anywhere else.
 *   - Based upon this, Remove knows that the project from which it should
 *     remove the object has to be the first (and only) one in the list.
 *
 */ 

DlgObject *DlgObject::NewObject( ProjectWnd &prjWnd, const QString &fileName )
{
  // Read first line to get type of file (dlg or xml)
  QFile inFile( fileName );
  if( ! inFile.open( IO_ReadOnly ) )
    return 0;
  QTextStream stream( &inFile );
  QString line = stream.readLine();
  inFile.close();

  QString type;
  DlgObject *retObj = NULL;

  bool asXML = FALSE;
  
  if( line == "<?xml version='1.0'?>" )
  {
    QtArchExpat parser( fileName );
    type = parser.GetType();
    asXML = TRUE;
  } else
  {
    DlgParser *itsParser = new DlgParser( fileName );

    if( ! itsParser->IsInitialized() )
      return NULL;

    type = itsParser->GetObjectType();
    asXML = FALSE;
  }

  if( type == "Dialog" )
    retObj = new Dialog( prjWnd, "" );
  else if( type == "Project" )
    retObj = new Project( prjWnd, "" );
  else if( type == "Pixmap" )
    retObj = new Pixmap( prjWnd, "" );

  retObj->fileTypeXML = asXML;
  
  return retObj;
}

QPopupMenu &DlgObject::GetListPopup()
{
  QPopupMenu *retPopup = NULL;

  if( InProject() )
  {
    if( status == DlgObject_Hidden )
      retPopup = boundHiddenPopup;
    else
      retPopup = boundPopup;
  } else
  {
    if( status == DlgObject_Hidden )
      retPopup = unboundHiddenPopup;
    else
      retPopup = unboundPopup;
  }

  if( ! retPopup )
    fatal( 
        tr( "DlgObject (Type %1) misconfigured, %2 in Project, status = %3" ).
        arg( (int)GetObjType() ).
        arg( (InProject() ? "" : "not") ).
        arg( (int)status ) );
           
  return *retPopup;
}

bool DlgObject::Remove()
{
  // XXXX Until I'm not absolut clear if this new style casts work OK under
  //      other c++ compilers, we have to leave them out, sigh...
  //  Project *prj = dynamic_cast<Project*>( itsProjects.first() );

  Project *prj = (Project *) itsProjects.first();
  if( ! prj )
    return FALSE;

  prj->RemoveFromProject( this );

  itsProjects.remove( prj );
  return TRUE;
}

bool DlgObject::SaveAs()
{
  QString name = QFileDialog::getSaveFileName( QString::null, 
					       "*." + GetExtension() );

  if( name.isEmpty() )
    return FALSE;

  SetName( name );

  return Save();
}

bool DlgObject::EnterProject( const Project *prj )
{
  bool retVal = TRUE;

  if( itsProjects.findRef( prj ) == -1 )
    itsProjects.append( prj );
  else
    retVal = FALSE;

  return retVal;
}

bool DlgObject::LeaveProject( Project *prj )
{
  if( prj )
  {
    prj->RemoveFromProject( this );
    return itsProjects.remove( prj );
  } else
  {
    DlgObject *prj;

    for( prj = itsProjects.first();
         prj;
         prj = itsProjects.next() )
    {
      ((Project *)prj)->RemoveFromProject( this );
    }

    itsProjects.clear();
  }

  return TRUE;
}

int DlgObject::InProject( const Project *prj )
{
  if( prj )
    return ((itsProjects.findRef( prj ) != -1) ? 1 : 0 );
  else
    return itsProjects.count();

  return 0;
}

DlgObject::DlgObject( ProjectWnd &prj, const QString &name )
  : QObject( NULL, name ),
    itsProjectWnd( prj ),
    needsSave( FALSE )
{
  unboundPopup = NULL;
  unboundHiddenPopup = NULL;
  boundPopup = NULL;
  boundHiddenPopup = NULL;

  itsProjects.setAutoDelete( FALSE );

  status = DlgObject_Init;
}

DlgObject::~DlgObject()
{
  delete unboundPopup;
  delete boundPopup;
}


QString DlgObject::GetExtension( DlgObjectType	type )
{
    if( itsExtensionDict.isEmpty() )
    {
      itsExtensionDict.insert( DlgObject_Dialog, "dlg" );
      itsExtensionDict.insert( DlgObject_Pixmap, "pix" );
      itsExtensionDict.insert( DlgObject_Project, "prj" );
    }
    
    return itsExtensionDict[ type ];
}

void DlgObject::InitFileType()
{
  if( ! itsFilename.isEmpty() )
  {
    // Read first line to get type of file (dlg or xml)
    QFile inFile( itsFilename );
    if( ! inFile.open( IO_ReadOnly ) )
      return;
    QTextStream stream( &inFile );
    QString line = stream.readLine();
    inFile.close();

    if( line == "<?xml version='1.0'?>" )
      fileTypeXML = TRUE;
    else
      fileTypeXML = FALSE;
  }
}


void DlgObject::SetName( const QString &newName, bool change )
{
    itsFilename = newName;
    status = DlgObject_Changed;
    needsSave = TRUE;
    InitFileType();
    if( change )
        emit Changed( this );
}

