/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgObject.h,v 2.4 1999/12/02 02:22:47 jeff Exp $

*******************************************************************************/


#ifndef DlgObject_included
#define DlgObject_included

#include <qobject.h>
#include <qlist.h>
#include <qpopmenu.h>
#include <qintdict.h>

class DlgParser;
class ProjectWnd;
class Project;

enum DlgObjectType {
    DlgObject_Dialog,
    DlgObject_Project,
    DlgObject_Pixmap
};

class DlgObject : public QObject
{
    Q_OBJECT

public: // methods
    virtual QPopupMenu &GetListPopup();
    // Return popupmenu for DialogList

    virtual ~DlgObject();
    // dtor, destroys at least our lists and popups

    virtual QString GetType() = 0;
    virtual DlgObjectType GetObjType() = 0;
    // Retrieves Object Type

    void InitFileType();
    // After setting the file name, InitFileType is able to distinguish
    // between old style and new style dlg files
    
    QString GetExtension();
    // Retrieves Filename extension

    static QString GetExtension( DlgObjectType type );
    // Get the extension for the object type

    const QString &GetName();
    virtual void SetName( const QString &, bool = TRUE );
    // get and set the filename

    static DlgObject *NewObject( ProjectWnd &, const QString & );
    // Build up a new object

    virtual void GetGeneratedFiles( QString &, QString &, QString & );
    // Get back the generated Sources, Headers and Headers which contains
    // Metastructure and need to be compiled with moc
    // Is needed for Makefilegeneration.

    QList<DlgObject> &GetProjects();
  // Returns a list of projects to which this obj belongs
  // List length can be retrieved by InProject()

    bool notSaved() const;
    // Has the DlgObject not been saved

public slots:
     virtual bool Load() = 0;
    // Load from disk

    virtual bool Save() = 0;
    // Save to disk

    virtual bool SaveAs();
    // Save to disk, different filename

    virtual bool Generate( bool ) = 0;
    virtual bool Generate();
    // Generate c++ files (the object must be shown/loaded or not)

    virtual void Hide() = 0;
    // Hide from screen

    virtual void Show() = 0;
    // Show (again)

    virtual void Raise() = 0;
    // Raise Window

    virtual void Delete() = 0;
    // Raise Window

    virtual bool EnterProject( const Project * );
    // Added in a project, remember this project

    virtual bool LeaveProject( Project * );
    // Removed from a Project, forget this project

    virtual bool Remove( );
    // Remove from a Project

    int InProject( const Project * = NULL );
    // Is in (specified) Project

    virtual void CPAvailable( bool ) = 0;
    // Somehow c&p data was made available

    signals:
    void Changed( DlgObject * );
    // DlgObject has changed, to be connected to DialogList

    void Closed( DlgObject * );
    // DlgObject was closed

    void Deleted( DlgObject * );
    // DlgObject is about to be deleted

    void Saved( DlgObject * );
    // Emitted after being saved

    void CPDataSet( DlgObject * );
    // Emitted after the c&p buffer was set by this object

protected: // Attributes
    ProjectWnd &itsProjectWnd;
    // Superclass

    DlgParser *itsParser;
    // Parser, used to read input files

    QList<DlgObject> itsProjects;
    // The list of loaded projects to which we belong

    QPopupMenu *unboundPopup;
    // Popup when not inside any project

    QPopupMenu *unboundHiddenPopup;
    // Popup when not inside any project, but hidden from screen

    QPopupMenu *boundPopup;
    // Popup when inside a project

    QPopupMenu *boundHiddenPopup;
    // Popup when inside a project, but hidden from screen

    QString itsFilename;
    // Filename to load/save

    bool fileTypeXML;
    // Set by DlgObject::NewObject after checking the filetype:
    // true: New style XML file
    // false: Old style dlg file
    
    enum Status {
	DlgObject_Init,
	DlgObject_Loaded,
	DlgObject_Saved,
	DlgObject_Generated,
	DlgObject_Shown,
	DlgObject_Hidden,
	DlgObject_Changed
    };

    Status status;
    bool needsSave;

protected: // Methods
    DlgObject( ProjectWnd &, const QString &name );
    // Ctor, name is passed further down to Qt


private: // Attributes

    static QIntDict<char>	itsExtensionDict;
    // dictionary of file extensions indexed by object type

};

typedef QList<DlgObject> DlgObjList;

inline const QString &DlgObject::GetName()
{
    return itsFilename;
}


inline QString DlgObject::GetExtension()
{
    return GetExtension( GetObjType() );
}


inline void DlgObject::GetGeneratedFiles( QString &src, 
                                          QString &hdr, 
                                          QString &mhdr )
{
    // We haven't got anything 
    src = "";
    hdr = "";
    mhdr = "";
}

inline bool DlgObject::Generate()
{
    return Generate( FALSE );
}

inline QList<DlgObject> &DlgObject::GetProjects()
{
  return itsProjects;
}

inline bool DlgObject::notSaved() const
{
    return needsSave;
}

#endif
