/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutWnd.h,v 2.3 1999/10/16 01:45:29 jeff Exp $

*******************************************************************************/


#ifndef LayoutWnd_included
#define LayoutWnd_included

#include <qaccel.h>
#include <qlistview.h>
#include <qmainwindow.h>
#include <qsplitter.h>

#include "layout/LayoutItem.h"
#include "layout/LayoutPropFrame.h"

class LayoutHelper;
class QToolButton;

class LayoutWnd : public QMainWindow
{
    Q_OBJECT

public: // Methods

    LayoutWnd( LayoutHelper&	theLayoutHelper,
	       QListView*&	layoutTree,
	       QWidget*		parent = NULL,
	       const char*	name = "LayoutWnd" );
    
    ~LayoutWnd();

    LayoutItem*	GetRootTreeItem() const;
    // Return the item at the root of the layout tree.
    
    LayoutItem* InsertRootLayout( 
	LayoutItem::LayoutItemType	newRootItem 
	);
    // Insert the given new root item as the root item in the tree

signals: // signals

    void ContentsChanged();
    // Signal emitted whenever the contents of the layout window have
    // changed.


    void Closed();
    // The window was closed

protected:

    virtual void closeEvent(
	QCloseEvent*	theCloseEvent
	);
    

private slots: // slots

    //
    // Menu item handlers
    //

    void EditDeleteItem();
    // Delete Current Item

    void InsertBoxLayout();
    // Insert a box layout

    void InsertBoxSpacing();
    // Insert spacing into the layout

    void InsertBoxStretch();
    // Insert stretch into the layout

    void InsertGridLayout();
    // Insert a grid layout

    void InsertGridRow();
    // Insert a row in a grid layout

    void InsertGridSpacer();
    // Insert a grid spacer

    void InsertLayoutWidget();
    // Insert a widget into the layout

    //
    // Toolbar item position handlers
    //

    void MoveItemUp();
    // Move the current item up the current branch

    void MoveItemDown();
    // Move the current item down the current branch

    void MoveItemLeft();
    // Move the current item to a sibling of its parent

    void MoveItemRight();
    // Move the current item in, creating a new branch

    //
    // Tree handlers
    //

    void TreeSelChanged( 
	QListViewItem*	theItem
	);
    // Tree item selection changed


    void TreeDoubleClicked(
        QListViewItem*  theItem
        );
    // Tree item double clicked


    //
    // Screen handlers
    //

    void ForceUpdateScreen();
    // Update screen elements and force the layout to be redrawn

    void Close();
    // Close the window

private: // Methods

    void UpdateScreen(
	bool	forceLayoutRecreate = FALSE
	);
    // Update screen elements

    bool UpdatePropertyPage();
    // Update the property page for the selected item.  Returns TRUE if the
    // layout properties were changed and the layout needs to be recreated.

    void InsertLayoutItem( 
	LayoutItem::LayoutItemType	newLayoutItem 
	);
    // Insert the given layout item as a child of the selected item

    bool NoLayoutDefined( void ) const;
    // Return true if there is no layout defined

    //
    // Screen Updating methods
    //

    void UpdateItemInsertion( LayoutItem*	currentItem );
    // Update menu and toolbar insertion items

    void UpdateItemMovement( LayoutItem*	currentItem );
    // Update item movement items

    bool CanItemBeChild( const LayoutItem*      item,
			 const LayoutItem*	parent );
    // Return true if the specified itemType can be a child of the
    // given parent

private: // Attributes

    LayoutHelper&	myLayoutHelper;
    // The layout helper this window belongs to

    QSplitter		mySplitter;
    // Splitter for the window

    QListView*		myLayoutTree;
    // Tree for the layout

    LayoutPropFrame*	myPropertyFrame;
    // Frame for properties

    QListViewItem*	myRootTreeItem;
    // Root item of the tree

    QListViewItem*	myPreviousItem;
    // The previous tree item

    //
    // Menu Ids
    //
    int		myInsertBoxLayoutMenuId;
    int		myInsertBoxSpacingMenuId;
    int		myInsertBoxStretchMenuId;
    int		myInsertGridLayoutMenuId;
    int		myInsertGridRowMenuId;
    int		myInsertGridSpacerMenuId;
    int		myInsertLayoutWidgetMenuId;

    //
    // Toolbar ids
    //
    QToolButton*	myInsertBoxLayoutTbId;
    QToolButton*	myInsertBoxSpacingTbId;
    QToolButton*	myInsertBoxStretchTbId;
    QToolButton*	myInsertGridLayoutTbId;
    QToolButton*	myInsertGridRowTbId;
    QToolButton*	myInsertGridSpacerTbId;
    QToolButton*	myInsertLayoutWidgetTbId;
    QToolButton*	myMoveUpTbId;
    QToolButton*	myMoveDownTbId;
    QToolButton*	myMoveLeftTbId;
    QToolButton*	myMoveRightTbId;

    //
    // Accelerator ids
    //
    int		myMoveUpAccelId;
    int		myMoveDownAccelId;
    int		myMoveLeftAccelId;
    int		myMoveRightAccelId;

    QAccel	myKbdAccel;
    // Keyboard accelerators.  The same id is used for the accelerator
    // as the toolbar.
};


inline LayoutItem* LayoutWnd::GetRootTreeItem() const
{
    return const_cast<LayoutItem*>
        (static_cast<const LayoutItem*> (myRootTreeItem));
}



#endif // LayoutWnd_included
