/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: PixmapWnd.cpp,v 2.4 1999/10/16 01:48:04 jeff Exp $

*******************************************************************************/


#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>

#include "pixmap/Pixmap.h"
#include "pixmap/PixmapWnd.h"
#include "pixmap/PixmapOption.h"
#include "pixmap/PixmapRotate.h"
#include "pixmap/ColorFrame.h"
#include "bmp/PixmapFactory.h"

Previewer::Previewer( QWidget *p )
  : QFrame( p ),
    contents( NULL )
{
  horScr = new QScrollBar( this );
  horScr->setOrientation( QScrollBar::Horizontal );
  horScr->hide();
  connect( horScr, SIGNAL(valueChanged(int)), this, SLOT(ScrBar(int)) );
  verScr = new QScrollBar( this );
  verScr->setOrientation( QScrollBar::Vertical );
  verScr->hide();
  connect( verScr, SIGNAL(valueChanged(int)), this, SLOT(ScrBar(int)) );
}

void Previewer::SetPixmap( QPixmap *p )
{
  contents = p;
  Resize();
}

void Previewer::ScrBar( int )
{
  repaint();
}

void Previewer::Resize()
{
  QSize cS = contents->size();

  // Positioning the scrollbars
  verScr->setGeometry( size().width() - SCRBAR_DIMENSION, 0,
                       SCRBAR_DIMENSION, size().height() - SCRBAR_DIMENSION );

  horScr->setGeometry( 0, size().height() - SCRBAR_DIMENSION, 
                       size().width() - SCRBAR_DIMENSION, SCRBAR_DIMENSION );

  int w = width() - SCRBAR_DIMENSION;
  if( cS.width() > width() )
  {
    horScr->show();
    horScr->setRange( 0, cS.width() - w );
  } else
  {
    horScr->hide();
    horScr->setRange( 0, 0 );
  }

  int h = height() - SCRBAR_DIMENSION;
  if( cS.height() > height() )
  {
    verScr->show();
    verScr->setRange( 0, cS.height() - h );
  } else
  {
    verScr->hide();
    verScr->setRange( 0, 0 );
  }

  QPainter p;
  p.begin( this );
  p.fillRect( rect(), backgroundColor() );
  p.end();

}

void Previewer::drawContents( QPainter * )
{
  if( ! contents )
    return;

  bitBlt( this, 0, 0, contents, 
          horScr->value(), verScr->value(), 
          (horScr->isVisible() ? width() - SCRBAR_DIMENSION : contents->width() ), 
          (verScr->isVisible() ? height() - SCRBAR_DIMENSION : contents->height() ) );
}

void Previewer::resizeEvent( QResizeEvent * )
{
  Resize();
}
 
PixmapWnd::PixmapWnd( Pixmap *p, ProjectWnd *prj, const char *name )
  : QMainWindow( NULL, name ),
    itsPixmapDraw( 0 ),
    pixmap( p ),
    itsModeGroup( 0 ),
    itsStatusZoom( 0 ),
    itsStatusPSize( 0 ),
    itsStatusPos( 0 ),
    itsStatusSize( 0 ),
    theProjectWnd( prj )
{
    setCaption( "Qt Architect" );

    setIcon( PixmapFactory::GetPixmap( QtArch_bmp ) );
    setIconText( caption() );

    // Setup main widget
    QSplitter* mainWidget = new QSplitter( this );
    itsPixmapDraw = new PixmapDraw( mainWidget );

    // Dialog knows what to do when we want to close
    connect( this, SIGNAL(Closed()), pixmap, SLOT(Hide()) );
  
    // Best for us if layout reports changes directly to pixmap
    connect( itsPixmapDraw, SIGNAL(Changed()), pixmap, SLOT(PixmapChanged()) );

    // setup menu items
    QPopupMenu* file = new QPopupMenu;
    file->insertItem( tr( "Save" ), pixmap, SLOT(Save()), CTRL+Key_S );
    file->insertItem( tr( "Generate Files" ), pixmap, SLOT(Generate()), CTRL+Key_G );
    file->insertSeparator();
    file->insertItem( tr( "Import..." ), this, SLOT(Import()) );
    file->insertItem( tr( "Export..." ), this, SLOT(Export()) );
    file->insertSeparator();
    file->insertItem( tr( "Options..." ), this, SLOT(SetOptions()) );
    file->insertSeparator();
    file->insertItem( tr( "Close" ), this, SLOT(Close()) );

    edit = new QPopupMenu;
    edit->insertItem( tr( "Select" ), this, SLOT(Select()) );
    deselectEntry = edit->insertItem( tr( "Deselect" ), this, SLOT(Deselect()) );
    edit->insertSeparator();
    clipEntry = edit->insertItem( tr( "Clip Selection" ), this, SLOT(Clip()) );
    edit->insertSeparator();
    cutEntry   = edit->insertItem( tr( "Cut" ), this, SLOT(Cut()) );
    copyEntry  = edit->insertItem( tr( "Copy" ), this, SLOT(Copy()) );
    pasteEntry = edit->insertItem( tr( "Paste" ), itsPixmapDraw, SLOT(Paste()) );
    edit->setItemEnabled( deselectEntry, FALSE );
    edit->setItemEnabled( clipEntry, FALSE );
    edit->setItemEnabled( cutEntry, FALSE );
    edit->setItemEnabled( copyEntry, FALSE );
    edit->setItemEnabled( pasteEntry, FALSE );
    edit->setCheckable( TRUE );

    modes = new QPopupMenu;
    modes->insertItem( tr( "Point" ), PixmapDraw::DrawPoint );
    modes->insertItem( tr( "Line" ), PixmapDraw::DrawLine );
    modes->insertItem( tr( "Rectangle" ), PixmapDraw::DrawRect );
    modes->insertItem( tr( "Circle" ), PixmapDraw::DrawCircle );
    modes->insertItem( tr( "Fill" ), PixmapDraw::DrawFill );
    modes->setCheckable( TRUE );

    tools = new QPopupMenu;
    tools->insertItem( tr( "Rotate..." ), this, SLOT(Rotate()) );
    useGridMenu = tools->insertItem( tr( "Grid" ), this, SLOT(UseGrid()) );
    tools->setCheckable( TRUE );
    tools->insertSeparator();

    sizes = new QPopupMenu;
    sizes->insertItem( "1", 1 );
    sizes->insertItem( "2", 2 );
    sizes->insertItem( "4", 4 );
    sizes->insertItem( "8", 8 );
    sizes->insertItem( tr( "Other..." ), 0 );
    sizes->setCheckable( TRUE );
    connect( sizes, SIGNAL(activated(int)), this, SLOT(PixelSize(int)) );
    tools->insertItem( tr( "Pixel size" ), sizes );

    styles = new QPopupMenu;
    styles->insertItem( tr( "Solid" ), SolidLine );
    styles->insertItem( tr( "Dashed" ), DashLine );
    styles->insertItem( tr( "Dot" ), DotLine );
    styles->insertItem( tr( "Dash Dot" ), DashDotLine );
    styles->insertItem( tr( "Dash Dot Dot" ), DashDotDotLine );
    styles->setCheckable( TRUE );
    connect( styles, SIGNAL(activated(int)), this, SLOT(LineStyles(int)) );
    tools->insertItem( tr( "Line style" ), styles );

    QMenuBar& menu = *menuBar();
    menu.insertItem( tr( "Pixmap" ), file );
    menu.insertItem( tr( "Edit" ), edit );
    menu.insertItem( tr( "Draw" ), modes );
    menu.insertItem( tr( "Tool" ), tools );

    // Connect the drawing modes
    connect( modes, SIGNAL(activated(int)), this, SLOT(ModeChange(int)) );

    // connect some links from PixmapDraw to us
    connect( itsPixmapDraw, SIGNAL(MouseTracking(int,int,int,int)), 
	     this, SLOT(MouseTracking(int,int,int,int)) );
    connect( itsPixmapDraw, SIGNAL(CPAvailable()), 
	     this, SLOT(CPAvailable()) );


    // Setup Toolbar
    QToolBar* toolBar = new QToolBar( "Toolbar", this );

    itsModeGroup = new QButtonGroup( this );
    itsModeGroup->setExclusive( TRUE );
    QToolButton* toolButton = 0;

    toolButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( DrawPixel_bmp ),
				   QIconSet::Small ),
			 tr( "Draw Pixel" ),
			 tr( "Draw single pixels" ),
			 0,
			 0,
			 toolBar );
    toolButton->setToggleButton( TRUE );
    itsModeGroup->insert( toolButton, PixmapDraw::DrawPoint );

    toolButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( DrawLine_bmp ),
				   QIconSet::Small ),
			 tr( "Draw Line" ),
			 tr( "Draw a line" ),
			 0,
			 0,
			 toolBar );
    toolButton->setToggleButton( TRUE );
    itsModeGroup->insert( toolButton, PixmapDraw::DrawLine );

    toolButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( DrawRect_bmp ),
				   QIconSet::Small ),
			 tr( "Draw Rectangle" ),
			 tr( "Draw a rectangle" ),
			 0,
			 0,
			 toolBar );
    toolButton->setToggleButton( TRUE );
    itsModeGroup->insert( toolButton, PixmapDraw::DrawRect );

    toolButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( DrawCircle_bmp ),
				   QIconSet::Small ),
			 tr( "Draw Circle" ),
			 tr( "Draw a circle" ),
			 0,
			 0,
			 toolBar );
    toolButton->setToggleButton( TRUE );
    itsModeGroup->insert( toolButton, PixmapDraw::DrawCircle );

    toolButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( DrawFill_bmp ),
				   QIconSet::Small ),
			 tr( "Fill" ),
			 tr( "Fill area" ),
			 0,
			 0,
			 toolBar );
    toolButton->setToggleButton( TRUE );
    itsModeGroup->insert( toolButton, PixmapDraw::DrawFill );

    QToolBar* zoomToolBar = new QToolBar( "Zoom", this );

    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( PixmapZoomIn_bmp ),
				      QIconSet::Small ),	
			    tr( "Zoom in" ),
			    tr( "Zoom into Pixmap" ),
			    itsPixmapDraw,
			    SLOT(ZoomIn()),
			    zoomToolBar );
    
    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( PixmapZoomOut_bmp ),
				      QIconSet::Small ),	
			    tr( "Zoom out" ),
			    tr( "Zoom out of Pixmap" ),
			    itsPixmapDraw,
			    SLOT(ZoomOut()),
			    zoomToolBar );

    connect( itsPixmapDraw, SIGNAL(ZoomChanged(int)), 
	     this, SLOT(ZoomChanged(int)));

    connect( itsModeGroup, SIGNAL(clicked(int)), this, SLOT(ModeChange(int)) );

    //
    // Setup main widget
    //

    itsPixmapDraw->setMinimumSize( 400, 300 );
  
    QSplitter* splitterRhs = new QSplitter( QSplitter::Vertical, mainWidget );

    // Set up color choosing
    itsColorArea = new ColorFrame( splitterRhs );
    itsColorArea->setMinimumSize( 200, 200 );
    itsColorArea->setFrameStyle( QFrame::Panel | QFrame::Raised );
    connect( itsColorArea, SIGNAL(ColorChanged(int,const QColor &)),
	     this, SLOT(ColorChanged(int,const QColor &)) );

    // Set up previewer
    itsPreviewArea = new Previewer( splitterRhs );
    itsPreviewArea->setMinimumSize( 200, 100 );
    itsPreviewArea->setFrameStyle( QFrame::Panel | QFrame::Raised );
    itsPixmapDraw->SetupPreview( itsPreviewArea );

    setCentralWidget( mainWidget );

    // setup status bar
    QStatusBar& statusbar = *statusBar();

    itsStatusZoom = new QLabel( &statusbar, tr( "Zoom" ) );
    statusbar.addWidget( itsStatusZoom, 2 );
    
    itsStatusPSize = new QLabel( &statusbar, tr( "PSize" ) );
    statusbar.addWidget( itsStatusPSize, 1, TRUE );
    
    itsStatusPos = new QLabel( &statusbar, tr( "Pos" ) );
    statusbar.addWidget( itsStatusPos, 1, TRUE );
    
    itsStatusSize = new QLabel( &statusbar, tr( "Size" ) );
    statusbar.addWidget( itsStatusSize, 1, TRUE );

    // Set initial drawing area
    ModeChange( PixmapDraw::DrawPoint );

    sizes->setItemChecked( 1, TRUE );
    itsPixmapDraw->SetPixelSize( 1 );
    styles->setItemChecked( SolidLine, TRUE );
    itsPixmapDraw->SetLineStyle( SolidLine );

    itsPixmapDraw->SetSize( QSize( 32, 32 ) );
    itsPixmapDraw->SetLeftColor( black );
    itsColorArea->SetLeftColor( black );
    itsPixmapDraw->SetRightColor( white );
    itsColorArea->SetRightColor( white );

    // resize to minimum size
    resize( 600, 400 );
}

PixmapWnd::~PixmapWnd()
{
  delete itsPixmapDraw;
}

void PixmapWnd::closeEvent( QCloseEvent*	event )
{
  // shut up
  event = event;

  emit Closed();
}

void PixmapWnd::resizeEvent( QResizeEvent *event )
{

  //  itsPixmapDraw->resize( 
  QMainWindow::resizeEvent( event );
}

void PixmapWnd::Close()
{
  emit Closed();
}

void PixmapWnd::Import()
{
  QString file = QFileDialog::getOpenFileName();

  if( ! file.isEmpty() )
    itsPixmapDraw->LoadFile( file );
}

void PixmapWnd::Export()
{
  QString file = QFileDialog::getSaveFileName();

  if( ! file.isEmpty() )
    itsPixmapDraw->WriteFile( file, "BMP" );
}

void PixmapWnd::SetOptions()
{
  // Collect some data
  const QColor &actBackground = itsPixmapDraw->GetBackgroundColor();
  QSize actSize = itsPixmapDraw->GetSize();

  // Display data
  PixmapOption *dlg = new PixmapOption( actBackground, actSize, this );

  // Write data
  if( dlg->exec() )
  {
    if( actBackground != dlg->GetBackgroundColor() )
    {
      itsPixmapDraw->SetBackgroundColor( dlg->GetBackgroundColor() );
    }
    QSize size = dlg->GetSize();
    itsPixmapDraw->SetSize( size );

    QString str = 
        tr( "Width:%1 Height:%2" ).
        arg( size.width(), 3 ).
        arg( size.height(), 3 );

    itsStatusPSize->setText( str );
  }

  delete dlg;
}

void PixmapWnd::ModeChange( int id )
{
    if( id >= PixmapDraw::LastDrawMode ||
	id < PixmapDraw::DrawPoint )
	return;

    QToolButton* button = static_cast<QToolButton*>(itsModeGroup->find( id ));
    if( button )
    {
	button->setDown( TRUE );
    }

    // Uncheck all items
    for( unsigned int i = 1; i < modes->count() + 1; i++ )
	modes->setItemChecked( i, FALSE );
    // Check the new item
    modes->setItemChecked( id, TRUE );

    itsPixmapDraw->ModeChange( (PixmapDraw::DrawMode) id );
}

void PixmapWnd::PixelSize( int id )
{
  // Uncheck all items
  int item = 1;
  for( unsigned int i = 1; i < sizes->count() + 1; i++, item *= 2 )
    sizes->setItemChecked( item, FALSE );
  // Check the new item
  sizes->setItemChecked( id, TRUE );

  if( id == 0 )
  {
    id = 1;
  }

  itsPixmapDraw->SetPixelSize( id );
}

void PixmapWnd::LineStyles( int id )
{
  // Uncheck all items
  for( unsigned int i = 1; i < styles->count() + 1; i++ )
    styles->setItemChecked( i, FALSE );
  // Check the new item
  styles->setItemChecked( id, TRUE );

  itsPixmapDraw->SetLineStyle( id );
}

void PixmapWnd::Rotate()
{
  PixmapRotate *dlg = new PixmapRotate( this );

  if( dlg->exec() )
  {
    // First rotation
    itsPixmapDraw->Rotate( dlg->GetRotation() );

    // Then eventually flip
    if( dlg->FlipVertical() )
      itsPixmapDraw->Flip( PixmapDraw::Vertical );
    if( dlg->FlipHorizontal() )
      itsPixmapDraw->Flip( PixmapDraw::Horizontal );
  }
}

void PixmapWnd::UseGrid()
{
  tools->setItemChecked( useGridMenu, ! tools->isItemChecked( useGridMenu ) );
  itsPixmapDraw->UseGrid( tools->isItemChecked( useGridMenu ));
}

void PixmapWnd::MouseTracking( int x, int y,
                               int width, int height )
{
  itsStatusPos->setText( tr( "X:%1 Y:%1" ).arg( x, 3 ).arg( y, 3 ) );

  if( width || height )
  {
    itsStatusSize->setText( tr( "W:%1 H:%2" ).arg( width, 3 ).arg( height, 3 ) );
  }
}

void PixmapWnd::ColorChanged( int button, const QColor &c )
{
  if( button == LeftButton )
    itsPixmapDraw->SetLeftColor( c );
  else if( button == RightButton )
    itsPixmapDraw->SetRightColor( c );
}

void PixmapWnd::ZoomChanged( int zoom )
{
  itsStatusZoom->setText( tr( "Zoom: %1").arg( zoom ) );
}


void PixmapWnd::Select()
{
  ModeChange( PixmapDraw::DrawRegion );
}

void PixmapWnd::Deselect()
{
  itsPixmapDraw->Deselect();
  edit->setItemEnabled( deselectEntry, FALSE );
  edit->setItemEnabled( clipEntry, FALSE );
  edit->setItemEnabled( cutEntry, FALSE );
  edit->setItemEnabled( copyEntry, FALSE );
  edit->setItemEnabled( pasteEntry, FALSE );

  edit->setItemChecked( clipEntry, FALSE );
  ModeChange( PixmapDraw::DrawPoint );
}

void PixmapWnd::Clip()
{
  edit->setItemChecked( clipEntry, ! edit->isItemChecked( clipEntry ) );
  itsPixmapDraw->ClipRegion();
}

void PixmapWnd::CPAvailable()
{
  edit->setItemEnabled( deselectEntry, TRUE );
  edit->setItemEnabled( clipEntry, TRUE );
  edit->setItemEnabled( cutEntry, TRUE );
  edit->setItemEnabled( copyEntry, TRUE );
}

void PixmapWnd::Cut()
{
  if( itsPixmapDraw->Cut() )
    edit->setItemEnabled( pasteEntry, TRUE );
}

void PixmapWnd::Copy()
{
  if( itsPixmapDraw->Copy() )
    edit->setItemEnabled( pasteEntry, TRUE );
}




