/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgLineEdit.cpp,v 2.9 2000/04/30 17:09:47 jeff Exp $

*******************************************************************************/


#include "widget/DlgLineEdit.h"

#include "prop/LineEditPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgLineEdit::itsSignals;
int DlgLineEdit::itsNextWidgetNum = 1;

DlgLineEdit::DlgLineEdit() :
    Inherited( DlgWidget::DW_LineEdit )
{
}


DlgLineEdit::DlgLineEdit(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgLineEdit::~DlgLineEdit()
{
}


void DlgLineEdit::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new LineEditPage( this, parent, "LineEdit" ) );

    labels.append( tr( "Line Edit" ) );
}

void DlgLineEdit::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "LineEdit" );

  QLineEdit* widget = GetLineEdit();

  generator.WriteSingleLine( "Text", widget->text() );
  generator.WriteSingleLine( "EchoMode",
                             EchoModeToStr( widget->echoMode() ) );
  generator.WriteXMLValue( "MaxLength", widget->maxLength() );
  generator.WriteXMLValue( "FrameShown", widget->frame() );

  generator.WriteSingleLine( "Alignement",
                             AlignmentToStr( widget->alignment() ) );
  
  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgLineEdit::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QLineEdit* widget = GetLineEdit();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else if( key == "MaxLength" )
    {
	widget->setMaxLength( value.toInt() );
    }
    else if( key == "EchoMode" )
    {
	widget->setEchoMode( StrToEchoMode( value ) );
    }
    else if( key == "FrameShown" )
    {
	widget->setFrame( (value == "TRUE") );
    }
    else if( key == "Alignment" )
    {
        widget->setAlignment( StrToAlignment( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgLineEdit::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "LineEdit" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetLineEdit() )
    {
        CreateQtWidget( 0 );
    }
  
    QLineEdit* widget = GetLineEdit();

    widget->setText( map["Text"].AsQString() );
    widget->setMaxLength( map["MaxLength"].AsInt() );
    widget->setEchoMode( StrToEchoMode( map["EchoMode"].AsQString() ) );

    widget->setFrame( map["FrameShown"].AsBool() );
    widget->setAlignment( StrToAlignment( map["Alignment"].AsQString() ) );

    // We inherit from 
    Inherited::RestoreXML( map["DlgWidget"] );

}


void DlgLineEdit::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QLineEdit* widget = GetLineEdit();

    stream << varName << "setText( ";
    GenerateText( widget->text(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << 
	"setMaxLength( " << widget->maxLength() << " );" << endl;
    
    stream << varName << "setFrame( QLineEdit::" <<
	EchoModeToStr( widget->echoMode() ) << " );" << endl;
    
    stream << varName << "setFrame( " <<
	(widget->frame() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAlignment( " <<
        AlignmentToStr( widget->alignment() ) << " );" << endl;
}


void DlgLineEdit::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgLineEdit::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "textChanged(const QString&)" );
	itsSignals.append( "returnPressed()" );
    }

    return itsSignals;
}


QString DlgLineEdit::EchoModeToStr( QLineEdit::EchoMode mode )
{
    QString str= "Normal";
    
    switch( mode )
    {
	case QLineEdit::Normal:
	default:
	{
	    // already set to Normal
	    break;
	}
	case QLineEdit::NoEcho:
	{
	    str = "NoEcho";
	    break;
	}
	case QLineEdit::Password:
	{
	    str = "Password";
	    break;
	}
    }
    
    return str;
}


QLineEdit::EchoMode DlgLineEdit::StrToEchoMode( const QString& str )
{
    QLineEdit::EchoMode echo = QLineEdit::Normal;
    
    if( str == "NoEcho" )
    {
	echo = QLineEdit::NoEcho;
    }
    else if( str == "Password" )
    {
	echo = QLineEdit::Password;
    }
    
    return echo;
}


void DlgLineEdit::CreateQtWidget(
    const char*	theName
    )
{
    QLineEdit* widget = 
	new QLineEdit( GetParent(),
		       CreateName( "LineEdit", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget );
}
