/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgListBox.cpp,v 2.9 2000/04/30 17:09:48 jeff Exp $

*******************************************************************************/


#include "widget/DlgListBox.h"

#include "prop/ListBoxPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgScrollView

QStrList DlgListBox::itsSignals;
int DlgListBox::itsNextWidgetNum = 1;

DlgListBox::DlgListBox() :
    Inherited( DlgWidget::DW_ListBox )
{
}


DlgListBox::DlgListBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgListBox::~DlgListBox()
{
}


void DlgListBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ListBoxPage( this, parent, "ListBox" ) );

    labels.append( tr( "List Box" ) );
}

void DlgListBox::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "ListBox" );
  
  QListBox* widget = GetListBox();

  generator.WriteStartList( "ItemList" );
  for( unsigned int i = 0; i < widget->count(); ++i )
    generator.WriteSingleLine( "Item", widget->text( i ) );
  generator.WriteEndBlock();
  
  generator.WriteXMLValue( "SelectionMode", 
                           SelectionModeToStr( widget->selectionMode() ) );

  generator.WriteXMLValue( "ColumnMode",
                           LayoutModeToStr( widget->columnMode() ) );
  
  if( widget->columnMode() == QListBox::FixedNumber )
  {
      generator.WriteXMLValue( "ColumnModeNum", widget->numColumns() );
  }

  generator.WriteXMLValue( "RowMode", LayoutModeToStr( widget->rowMode() ) );

  if( widget->rowMode() == QListBox::FixedNumber )
  {
      generator.WriteXMLValue( "RowModeNum", widget->numRows() );
  }
  
  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgListBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QListBox* widget = GetListBox();

    // Compare to known keys
    if( key == "Item" )
    {
	widget->insertItem( value );
    }
    else if( key == "DragSelect" )
    {
        // Deprecated
    }
    else if( key == "AutoScroll" )
    {
        // Deprecated
    }
    else if( key == "ScrollBar" )
    {
	widget->setVScrollBarMode( 
            (value == "TRUE") ? QListBox::AlwaysOn : QListBox::AlwaysOff );
    }
    else if( key == "AutoScrollBar" )
    {
	widget->setVScrollBarMode( 
            (value == "TRUE") ? QListBox::Auto : QListBox::AlwaysOff );
    }
    else if( key == "BottomScrollBar" )
    {
	widget->setHScrollBarMode(
            (value == "TRUE") ? QListBox::AlwaysOn : QListBox::AlwaysOff );
    }
    else if( key == "AutoBottomScrollBar" )
    {
	widget->setHScrollBarMode(
            (value == "TRUE") ? QListBox::Auto : QListBox::AlwaysOff );
    }
    else if( key == "SmoothScrolling" )
    {
        // Deprecated
    }
    else if( key == "MultiSelection" )
    {
	widget->setMultiSelection( (value == "TRUE") );
    }
    else if( key == "AutoUpdate" )
    {
        // Deprecated
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgListBox::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "ListBox" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetListBox() )
    {
        CreateQtWidget( 0 );
    }
  
    QListBox* widget = GetListBox();

    XMLValueList list = map["ItemList"].AsList();
    QListIterator<XMLValue> it( list );
    for( ; it.current(); ++it )
    {
        widget->insertItem( it.current()->AsQString() );
    }
    
    widget->setSelectionMode( 
        StrToSelectionMode( map["SelectionMode"].AsQString() ) );
  
    QListBox::LayoutMode columnMode =  
        StrToLayoutMode( map["ColumnMode"].AsQString() );
    widget->setColumnMode( columnMode );
  
    if( columnMode == QListBox::FixedNumber )
    {
        widget->setColumnMode( map["ColumnModeNum"].AsInt() );
    }
  
    QListBox::LayoutMode rowMode = 
        StrToLayoutMode( map["RowMode"].AsQString() );
    widget->setRowMode( rowMode );
  
    if( rowMode == QListBox::FixedNumber )
    {
        widget->setRowMode( map["RowModeNum"].AsInt() );
    }

    // We inherit from 
    Inherited::RestoreXML( map["ScrollView"] );

}


void DlgListBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QListBox* widget = GetListBox();

    for( unsigned int i = 0; i < widget->count(); ++i )
    {
	stream << varName << "insertItem( ";
        GenerateText( widget->text( i ), widgetLayout, stream );
        stream << " );" << endl;
    }

    stream << varName << "setSelectionMode( QListBox::"
           << SelectionModeToStr( widget->selectionMode() )
           << " );" << endl;
    
    if( widget->columnMode() == QListBox::FixedNumber )
    {
        stream << varName << "setColumnMode( " << widget->numColumns() << " );"
               << endl;
    }
    else
    {
        stream << varName << "setColumnMode( QListBox::"
               << LayoutModeToStr( widget->columnMode() )
               << " );" << endl;
    }
    
    if( widget->rowMode() == QListBox::FixedNumber )
    {
        stream << varName << "setRowMode( " << widget->numRows() << " );"
               << endl;
    }
    else
    {
        stream << varName << "setRowMode( QListBox::"
               << LayoutModeToStr( widget->rowMode() )
               << " );" << endl;
    }
    
}


void DlgListBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgListBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "highlighted(int)" );
	itsSignals.append( "highlighted(const QString&)" );
	itsSignals.append( "highlighted(QListBoxItem *)" );
	itsSignals.append( "selected(int)" );
	itsSignals.append( "selected(const QString&)" );
	itsSignals.append( "selected(QListBoxItem *)" );
	itsSignals.append( "selectionChanged()" );
	itsSignals.append( "selectionChanged(QListBoxItem *)" );
	itsSignals.append( "currentChanged(QListBoxItem *)" );
	itsSignals.append( "clicked(QListBoxItem *)" );
	itsSignals.append( "clicked(QListBoxItem *,const QPoint &)" );
	itsSignals.append( "pressed(QListBoxItem *)" );
	itsSignals.append( "pressed(QListBoxItem *,const QPoint &)" );
	itsSignals.append( "doubleClicked(QListBoxItem *)" );
	itsSignals.append( "returnPressed(QListBoxItem *)" );
	itsSignals.append( "rightButtonClicked(QListBoxItem *,const QPoint &)" );
	itsSignals.append( "rightButtonPressed(QListBoxItem *,const QPoint &)" );
	itsSignals.append( "mouseButtonPressed(int,QListBoxItem *,const QPoint &)" );
	itsSignals.append( "mouseButtonClicked(int,QListBoxItem *,const QPoint &)" );
	itsSignals.append( "onItem(QListBoxItem *)" );
	itsSignals.append( "onViewport()" );
    }

    return itsSignals;
}


QString DlgListBox::SelectionModeToStr( QListBox::SelectionMode mode )
{
    QString selectionMode;
    switch( mode )
    {
        case QListBox::Single:
        {
            selectionMode = "Single";
            break;
        }
        case QListBox::Multi:
        {
            selectionMode = "Multi";
            break;
        }
        case QListBox::Extended:
        {
            selectionMode = "Extended";
            break;
        }
        case QListBox::NoSelection:
        {
            selectionMode = "NoSelection";
            break;
        }
    }
    
    return selectionMode;
}


QListBox::SelectionMode DlgListBox::StrToSelectionMode(
    const QString&  str 
    )
{
    QListBox::SelectionMode mode = QListBox::Single;
    if( str == "Single" )
    {
        mode = QListBox::Single;
    }
    else if( str == "Multi" )
    {
        mode = QListBox::Multi;
    }
    else if( str == "Extended" )
    {
        mode = QListBox::Extended;
    }
    else if( str == "NoSelection" )
    {
        mode = QListBox::NoSelection;
    }
    
    return mode;
}


QString DlgListBox::LayoutModeToStr( QListBox::LayoutMode mode )
{
    QString str;
    switch( mode )
    {
        case QListBox::FixedNumber:
        {
            str = "FixedNumber";
            break;
        }
        case QListBox::FitToWidth:
        {
            str = "FitToWidth";
            break;
        }
        /*
          case QListBox::FitToHeight:
        {       
        str = "FitToHeight";
        break;
        }
        */
        case QListBox::Variable:
        {
            str = "Variable";
            break;
        }
    }
    
    return str;
}


QListBox::LayoutMode DlgListBox::StrToLayoutMode( const QString& str )
{
    QListBox::LayoutMode mode = QListBox::Variable;
    if( str == "FixedNumber" )
    {
        mode = QListBox::FixedNumber;
    }
    else if( str == "FitToWidth" )
    {
        mode = QListBox::FitToWidth;
    }
    else if( str == "FitToHeight" )
    {
        mode = QListBox::FitToHeight;
    }
    else if( str == "Variable" )
    {
        mode = QListBox::Variable;
    }
    
    return mode;
}


void DlgListBox::CreateQtWidget(
    const char*	theName
    )
{
    QListBox* widget = 
	new QListBox( GetParent(),
		      CreateName( "ListBox", 
				  theName, 
				  itsNextWidgetNum ) );

    SetQtWidget( widget );
}
