/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgScrollView.cpp,v 2.7 2000/04/30 17:09:48 jeff Exp $

*******************************************************************************/


#include "widget/DlgScrollView.h"

#include "prop/ScrollViewPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgScrollView::itsSignals;
int DlgScrollView::itsNextWidgetNum = 1;

DlgScrollView::DlgScrollView() :
    Inherited( DlgWidget::DW_ScrollView )
{
}


DlgScrollView::DlgScrollView(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgScrollView::~DlgScrollView()
{
}


void DlgScrollView::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ScrollViewPage( this, parent, "ScrollView" ) );

    labels.append( tr( "Scroll View" ) );
}

void DlgScrollView::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "ScrollView" );
  
  QScrollView* widget = GetScrollView();

  generator.WriteSingleLine( "ResizePolicy", 
                             ResizePolicyToStr( widget->resizePolicy() ) );

    generator.WriteSingleLine( "VScrollBarMode", 
                               ScrollBarModeToStr( widget->vScrollBarMode() ) );
                               
    generator.WriteSingleLine( "HScrollBarMode", 
                               ScrollBarModeToStr( widget->hScrollBarMode() ) );

    generator.WriteXMLValue( "DragAutoScroll", widget->dragAutoScroll() );

    Inherited::Save( generator );

    generator.WriteEndBlock();
}


bool DlgScrollView::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QScrollView* widget = GetScrollView();

    // Compare to known keys
    if( key == "ResizePolicy" )
    {
	widget->setResizePolicy( StrToResizePolicy( value ) );
    } 
    else if( key == "VScrollBarMode" )
    {
	widget->setVScrollBarMode(
	    StrToScrollBarMode( value ) );
    }
    else if( key == "HScrollBarMode" )
    {
	widget->setHScrollBarMode(
	    StrToScrollBarMode( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgScrollView::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "ScrollView" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetScrollView() )
    {
        CreateQtWidget( 0 );
    }
  
    QScrollView* widget = GetScrollView();

    widget->setResizePolicy(
        StrToResizePolicy( map["ResizePolicy"].AsQString() ) );
    widget->setVScrollBarMode(
        StrToScrollBarMode( map["VScrollBarMode"].AsQString() ) );
    widget->setHScrollBarMode(
        StrToScrollBarMode( map["HScrollBarMode"].AsQString() ) );

    widget->setDragAutoScroll( map["DragAutoScroll"].AsBool() );

    // We inherit from 
    Inherited::RestoreXML( map["Frame"] );

}


void DlgScrollView::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QScrollView* widget = GetScrollView();

    stream << varName << "setResizePolicy( QScrollView::" <<
	ResizePolicyToStr( widget->resizePolicy() ) <<
	" );" << endl;
    
    stream << varName << "setVScrollBarMode( QScrollView::" <<
	ScrollBarModeToStr( widget->vScrollBarMode() ) <<
	" );" << endl;
    
    stream << varName << "setHScrollBarMode( QScrollView::" <<
	ScrollBarModeToStr( widget->hScrollBarMode() ) <<
	" );" << endl;    

    stream << varName << "setDragAutoScroll( "
           << (widget->dragAutoScroll() ? "TRUE" : "FALSE") 
           << " );" << endl;
}


void DlgScrollView::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgScrollView::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "contentsMoving(int,int)" );
    }

    return itsSignals;
}


QString DlgScrollView::ResizePolicyToStr(
    QScrollView::ResizePolicy thePolicy
    )
{
    QString policyStr;
    
    switch( thePolicy )
    {
	case QScrollView::Default:
	{
	    policyStr = "Default";
	    break;
	}
	case QScrollView::Manual:
	{
	    policyStr = "Manual";
	    break;
	}
	case QScrollView::AutoOne:
	{
	    policyStr = "AutoOne";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return policyStr;
}


QScrollView::ResizePolicy DlgScrollView::StrToResizePolicy(
    const QString& thePolicyStr
    )
{
    QScrollView::ResizePolicy policy = QScrollView::Default;
    
    if( thePolicyStr == "Default" )
    {
	policy = QScrollView::Default;
    }
    else if( thePolicyStr == "Manual" )
    {
	policy = QScrollView::Manual;
    }
    else if( thePolicyStr == "AutoOne" )
    {
	policy = QScrollView::AutoOne;
    }
    
    return policy;
}


QString DlgScrollView::ScrollBarModeToStr(
    QScrollView::ScrollBarMode theMode
    )
{
    QString modeStr;
    
    switch( theMode )
    {
	case QScrollView::Auto:
	{
	    modeStr = "Auto";
	    break;
	}
	case QScrollView::AlwaysOff:
	{
	    modeStr = "AlwaysOff";
	    break;
	}
	case QScrollView::AlwaysOn:
	{
	    modeStr = "AlwaysOn";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return modeStr;
}


QScrollView::ScrollBarMode DlgScrollView::StrToScrollBarMode(
    const QString& theModeStr
    )
{
    QScrollView::ScrollBarMode mode = QScrollView::Auto;
    
    if( theModeStr == "Auto" )
    {
	mode = QScrollView::Auto;
    }
    else if( theModeStr == "AlwaysOff" )
    {
	mode = QScrollView::AlwaysOff;
    }
    else if( theModeStr == "AlwaysOn" )
    {
	mode = QScrollView::AlwaysOn;
    }
    
    return mode;
}


void DlgScrollView::CreateQtWidget(
    const char*	theName
    )
{
    QScrollView* widget = 
	new QScrollView( GetParent(),
			 CreateName( "ScrollView", 
				     theName, 
				     itsNextWidgetNum ) );

    SetQtWidget( widget );
}
